import express         from 'express';
import reqGetDB           from './reqGetDB.js';
import db               from '../helpers/DatabaseHelper.js';
import { verifyJWT } from '../Middlewares.js';
import { checkUser } from "../calcFunctions.js"
import { MessageRoute } from './MessageRoute.js';
import { randomInt } from 'crypto';
import { calcNbPtsTotalQCM } from "../calcFunctions.js"


const router: express.Router = express.Router();


function shuffleArray<T>(array: T[]): T[] {
    for (let i = array.length - 1; i > 0; i--) {
        const j = randomInt(0, i);
        [array[i], array[j]] = [array[j], array[i]];
    }
    return array;
}

async function getCreatedQCMs(USER_ID: number)
{
    const qcms = await db.qcmTable.findMany({
        where: {
            idUserCreator: USER_ID
        },
        select: {
            idQCM: true,
            nomQCM: true,
            codeAcces: true,
        }
    });

    return Promise.all(
        qcms.map(async qcm => {
            const nbParticipants = await db.participer.count({
                where: {
                    idQCM: qcm.idQCM
                }
            });
            return {
                ...qcm,
                nbParticipants
            };
        })
    );
}

async function getQCMExamen(QCM_ID: number, res: express.Response) {
    const qcmDetails = await db.qcmTable.findUnique({
        where: { idQCM: QCM_ID },
        select: {
            nomQCM: true,
            temps: true,
            randomOrder: true,
            questions: {
                select: {
                    isMultiple: true,
                    idQuestion: true,
                    question: true,
                    randomOrder: true,
                    type: {
                        select: {
                            nomType: true
                        }
                    },
                    nbPtsPositif: true,
                    nbPtsNegatif: true,
                    choix: {
                        select: {
                            idChoix: true,
                            nomChoix: true
                        }
                    }
                    
                }
            }
        }
    });
    if (!qcmDetails)
    {
        res.status(404).send({error : MessageRoute.qcmDoesntExist});
    }
    else {
        console.log(qcmDetails)
        if(qcmDetails.randomOrder){
            //randomiser l'ordre des questions
            qcmDetails.questions = shuffleArray(qcmDetails.questions);
        }
        qcmDetails.questions.forEach(quest => {
            if(quest.randomOrder){
                quest.choix = shuffleArray(quest.choix);
            }
        })
    }
    return qcmDetails;
}

async function getInfosQCM(idQCM: number)
{
    const infosQCM = await db.qcmTable.findFirst({
        where: {
            idQCM: idQCM
        },
        include: {
            questions: {
                include: {
                    choix: true,
                    type: true
                }
            }
        }
    });
    console.table(infosQCM);
    return infosQCM;
}

async function getRealisedQCMs(USER_ID: number) 
{
    const participations = await db.participer.findMany({
        where: {
          idUser: USER_ID,   
          hasFinished: true,   
        },
        select: {
            idQCM: true,
            feedback: true,
            note: true,
            score: true,
            qcm: {
                select: {
                    nomQCM: true,
                    idQCM: true,  
                    questions: {
                        select: {
                            idQuestion: true,
                        }
                    },

                },
            },
            
        },
    });

    const participationsWithScoreMax = await Promise.all(participations.map(async participation => {
        const scoreMax = await calcNbPtsTotalQCM(participation.qcm.idQCM);
        console.log(scoreMax)
        return {
            ...participation,
            scoreMax,
        };
    }));
    console.table(participationsWithScoreMax)
    return participationsWithScoreMax;
}


router.get('/realised_QCMs/:USER_ID',verifyJWT, async (req: express.Request, res: express.Response) => {
    const USER_ID: number = parseInt(req.params.USER_ID, 10);
    checkUser(req, res, USER_ID, "You can't acces this ressource.");
    const qcms = await reqGetDB(() => getRealisedQCMs(USER_ID));
    res.send(qcms);
});

router.get('/created_QCMs/:USER_ID', verifyJWT, async (req: express.Request, res: express.Response) => {
    const USER_ID: number = parseInt(req.params.USER_ID, 10);
    checkUser(req, res, USER_ID, "You can't access this ressource.");
    const qcms = await reqGetDB(() => getCreatedQCMs(USER_ID));
    res.send(qcms);
});

router.get('/examen_QCM/:QCM_ID', verifyJWT, async (req: express.Request, res: express.Response) => {
    const QCM_ID: number = parseInt(req.params.QCM_ID, 10);
    if (req.user) {
        const { id } = req.user;    
        if (typeof id === 'number') {
            try {
                const qcm = await reqGetDB(() => getQCMExamen(QCM_ID, res));
                const qcmDetails = await db.participer.findUnique({
                    where: {
                        idUser_idQCM: {
                            idUser:id,
                            idQCM: QCM_ID
                        } 
                    },
                    select: {
                        heureDebut: true
                    }
                });

                if (!qcmDetails) {
                    res.status(404).json({ error: 'QCM details not found' });
                } else {
                    res.json({ qcm: qcm, heureDebut: qcmDetails.heureDebut });
                }
            } catch (error) {
                console.error(error);
                res.status(500).json({ error: 'Internal server error' });
            }
        } 
        else {
            res.status(400).json({ error: 'Invalid user ID' });
        }
    }
    else{
        res.status(401).send('Unauthorized');
    }
});

router.get('/infos_QCM/:QCM_ID', verifyJWT, async (req: express.Request, res: express.Response) => {
    const QCM_ID: number = parseInt(req.params.QCM_ID, 10);
    const results = await reqGetDB(() => getInfosQCM(QCM_ID));
    res.send(results);
});

export default router;
